#!/usr/bin/python
#Dependencies: sox libsox-fmt-all octave curl
#Additionally, if you're on python2.6: python-argparse python-imaging

import sys, subprocess, time, os, argparse, logging, random, operator
import solver, fuckit_splitter
import demo_downloader
from parse_solver_args import parse_solver_options,print_solver_help_and_exit

parser = argparse.ArgumentParser(description="Runs the complete captcha solving process with different values for the number of lowest captchas to find, in order to find the best value")

#Solver options
parser.add_argument('-s', nargs='*', action='append', help='Adds a solver to the list of solvers to be used to solve the captcha. ' 
                    + 'This option can be specified multiple times to use additional solvers as backup.  Use -sh for information on specifying solvers')
parser.add_argument('-sh', required=False, action='store_true', help='Show the syntax to specify solvers')

#Misc args
parser.add_argument('-a', required=False, action='store_true', help='Save debug images and audio files to help with debugging the splitter')
parser.add_argument('-l', type=int, required=False, default=sys.maxint, metavar='limit', help='The number of captchas to solve')
parser.add_argument('-m', required=False, action='store_true', help='Display timing information about the captchas solved')
parser.add_argument('-n', required=False, default=-1, type=int, metavar='number', help='Save the files used for solving captchas.  A number to start at when naming the captchas should be supplied as well.  They will be named according to the scheme: current####.')
parser.add_argument('-proxy_host', required=False, default="", type=str, metavar='proxy_host', help='Use the specified proxy host')
parser.add_argument('-proxy_port', required=False, default="8118", type=str, metavar='proxy_host', help='Use the specified proxy port')
parser.add_argument('-u', type=float, required=False, default=0, metavar='seconds', help='The minimum number of seconds to wait between downloading and submitting.')
parser.add_argument('-t', required=False, action='store_true', help='Use tor')
parser.add_argument('-w', type=int, required=False, default=0, metavar='seconds', help='The number of seconds to wait between submitting and downloading another.  Note, times will be +/- 50 percent to ensure randomness')
parser.add_argument('-v', required=False, action='store_true', help='Verbose mode')
parser.add_argument('-vv', required=False, action='store_true', help='Very Verbose mode')

args = parser.parse_args()
if args.sh:
    print_solver_help_and_exit()

output_level = logging.CRITICAL
if args.vv:
    output_level = logging.DEBUG
elif args.v:
    output_level = logging.INFO

masher = None #Until we get a working masher
splitter = fuckit_splitter.FuckItSplitter(debug = output_level)

downloaders = []
downloaders.append(demo_downloader.DemoDownloader(args.proxy_host, args.proxy_port, output_level))

solver_list, solver_thresholds = parse_solver_options(args.s, output_level)
captcha_solver = solver.CaptchaSolver(solver_list, splitter, downloaders, solver_thresholds, masher, output_level)

save_files = (args.n != -1)
if save_files:
    captcha_solver.make_folders()
    start = args.n
else:
    start = os.getpid()

correct = 0
streak = 0
combo = 0
i = start
while i <  start + args.l:
    if save_files:
        filename = "current" + str(i) 
    else:
        filename = "current" + str(start)

    challenge = ""

    startTime = time.time()
    challenge = captcha_solver.get_next_captcha(filename)
    if challenge == "":
        print "Could not download mp3 file",filename + ".mp3"
        continue
    doneDownloadingTime = time.time()

    answer = captcha_solver.solve(filename) #Solve the captcha word by word
    doneSolveTime = time.time()
    
    if answer == "":
        print "No answer found for",filename
        success = False
    else:
        seconds_to_wait = args.u - (time.time() - doneDownloadingTime)
        if seconds_to_wait > 0:
            print "Going to fast, waiting",seconds_to_wait,"before submitting"
            time.sleep(seconds_to_wait)
        doneWaitingTime = time.time()

        print "Submitting " + answer 
        success = captcha_solver.answer_captcha(challenge, answer)
        doneSubmittingTime = time.time()

        captcha_solver.feedback(success, answer, filename)
        if not save_files:
            captcha_solver.delete_files(success, filename)
        else:
            captcha_solver.save_files(success, answer, filename)  #Save all the resulting files

        if args.m:
            print "Time taken to get challenge and download: ", doneDownloadingTime - startTime
            print "Time taken to solve (and split if necessary) the captcha: ", doneSolveTime - doneDownloadingTime
            print "Time taken to submit the captcha: ", doneSubmittingTime - doneWaitingTime
            print "Total time: ", doneSubmittingTime - startTime

    if success:
        correct += 1
        print "Correct ",
        streak += 1
        print "(" + str(streak) + " in a row)",
        if streak > combo:
            combo = streak
    else:
        print "Incorrect ",
        streak = 0

    print "(Highscore " + str(combo) + ")",
    print correct,"/",i - start + 1," = ",round(float(correct)/float(i - start + 1) * 100, 2),"%"

    time.sleep(args.w * (random.random() + .5))
    i += 1

