import Image, os, subprocess, operator, logging, argparse, time, sys, math
from splitterbase import SplitterBase

class FuckItSplitter(SplitterBase):
    """Wraps all the methods required to split an audio recaptcha into the words using the pitch"""
    number_of_words_per_piece = 4
    number_of_pieces = 3
    def __init__(self, debug = logging.CRITICAL, debug_images = False, filename_prefix = "temp/", filename_suffix = ".mp3"):
        self.logger = logging.getLogger('FuckItSplitter')
        logging.basicConfig(format='%(asctime)s - %(name)s - %(levelname)s - %(message)s')
        self.logger.setLevel(debug)
        self.debug_images = debug_images

        self.filename_prefix = filename_prefix
        self.filename_suffix = filename_suffix
        self.image_index = {}

    def split_captcha(self, filename, split_components = [ "audio", "image" ]):
        """Splits a audio captcha and its pitch.  If only an audio file or only a spectrogram
            image file is needed for each word, split_components can be used to only select the component you need."""  
        self.image_index = {} #Reset the file nameing convention
        self.get_spectrogram(filename)
        spectrogram = self.filename_prefix + filename + ".png"
        image = Image.open(spectrogram).convert("RGB")
        image,box = self.remove_white_border(image)
            
        if self.debug_images:
            image.save(self.filename_prefix + filename + "_noborder.png", "PNG")
        pieces = self.split_into_pieces(image, self.number_of_pieces)

        words = []
        for piece in pieces:
            length = (piece[1] - piece[0])/self.number_of_words_per_piece
            for i in range(0,self.number_of_words_per_piece):
                words.append([piece[0] + length * i, piece[0] + length * (i+1)])
        self.logger.debug("Found %d words: %s", len(words), str(words))

        sizex, sizey = image.size
        if "audio" in split_components:
            self.split_wav_file(words, sizex, filename) 
        if "image" in split_components:
            for i in range(0, len(words)):
                cropped_image = image.crop((words[i][0], 0, words[i][1], sizey))
                cropped_image.save(self.filename_prefix + filename + "_" + str(i) + ".png", "PNG")

        return len(words)

    def split_into_pieces(self, image, number_of_pieces):
        minx = 0
        maxx = image.size[0]
        miny = 450
        maxy = 505
        mincolor = 249
        join = 50
        pixels = image.load()

        pieces = []
        for x in range(minx,maxx):
            for y in range(miny, maxy):
                if pixels[x,y][0] >= 250:
                    if len(pieces) == 0 or x - pieces[-1][1] > join:
                        pieces.append([x,x])
                        break
                    else:
                        pieces[-1][1] = x
                        break

        self.logger.debug("Found %d pieces: %s", len(pieces), pieces)
        if len(pieces) > number_of_pieces:
            lengths = {}
            for i in range(0,len(pieces)):
                lengths[i] = pieces[i][1] - pieces[i][0]
            lengths = sorted(lengths.iteritems(), key=operator.itemgetter(1))
            lengths.reverse()

            indexes = []
            for i in range(0, min(number_of_pieces, len(lengths))):
                indexes.append(lengths[i])
            indexes.sort()

            new_pieces = []
            for index in indexes:
                new_pieces.append(pieces[index[0]])
            pieces = new_pieces
             
            self.logger.debug("After removing thin ones found %d pieces: %s", len(pieces), pieces)
        while len(pieces) < number_of_pieces:
            lengths = {}
            for i in range(0,len(pieces)):
                lengths[i] = pieces[i][1] - pieces[i][0]
            lengths = sorted(lengths.iteritems(), key=operator.itemgetter(1))
            lengths.reverse()
            print lengths
            
            start = pieces[lengths[0][0]][0]
            end = pieces[lengths[0][0]][1]
            mid = start + (end - start) / 2
            pieces[lengths[0][0]][1] = mid
            pieces.insert(lengths[0][0] + 1, [mid, end])

            self.logger.debug("After splitting the thickest one found %d pieces: %s", len(pieces), pieces)

        return pieces

if __name__ == "__main__":    
    parser = argparse.ArgumentParser(description="Split an audio captcha into the individual words.  For the most of these options, you can mostly leave the defaults unless you feel like experimenting")
    parser.add_argument('filename', help='The audio file to split into individual words')
    parser.add_argument('-v', required=False, action='store_true', help='Verbose mode')
    parser.add_argument('-vv', required=False, action='store_true', help='Very Verbose mode')

    args = parser.parse_args()
    output_level = logging.CRITICAL
    debug_images = False
    if args.vv:
        output_level = logging.DEBUG
        debug_images = True
    elif args.v:
        output_level = logging.INFO

    dirname, filename = os.path.split(args.filename)
    if dirname != "":
        dirname += "/"
    extension = filename.split(".")[-1]
    filename = filename[:-1-len(extension)]

    splitter = FuckItSplitter(debug = output_level, debug_images = debug_images,filename_prefix = dirname, 
                                     filename_suffix = "." + extension)
    splitter.split_captcha(filename)



