import logging, time, subprocess, argparse, operator, math,sys
from solverbase import SolverBase

class HumanSolver(SolverBase):
    """This class will attempt to solve a captcha by letting a human type the answer"""

    classname = "HumanSolver"
    accuracy_multiplier = sys.maxint 
    split_components = []
    def __init__(self, debug = logging.CRITICAL, entire_thing = True, skip = 0.0, filename_prefix = "temp/", filename_suffix = ".wav"):
        """Initializes a solver that uses pHash to solve the captcha
            Parameters:
                debug = how much information should be printed
                filename_prefix = A filename prefix to add to the beginning of all filenames passed in
                filename_suffix = A filename suffix to add to the end of all the filenames passed in
        """
        self.logger = logging.getLogger(self.classname)
        logging.basicConfig(format='%(asctime)s - %(name)s - %(levelname)s - %(message)s')
        self.logger.setLevel(debug)
        
        if not entire_thing:
            self.split_components.append("audio")
        self.skip = skip

        self.filename_prefix = filename_prefix
        self.filename_suffix = filename_suffix
        self.mplayer = None

    def reset_solver(self):
        """Resets the state of the human solver"""
        self.filenames = []

    def start_solving(self, filename, number_of_words = 10):
        """Starts solving an entire captcha using phash"""
        self.reset_solver()
        for i in range(0, number_of_words): #Start the find_cluster processes
            self.start_solving_word(self.filename_prefix + filename + "_" + str(i) + self.filename_suffix)

    def finish_solving(self, filename):
        """Finishes solving the entire captcha by human"""
        predMatrices = {}
        if len(self.split_components) == 0:
            answer = ""
            filepath = self.filename_prefix + filename + ".mp3"
            while answer == "" or (answer.isdigit() and len(answer) < 3):
                self.play_file(filepath, self.skip)
                answer = raw_input(filepath + ") Enter the words you hear.  (Hit enter to replay it, or a number to replay starting that many seconds into the captcha)")
            parts = answer.split(" ")
            for i in range(0, len(parts)):
                predMatrices[i] = {parts[i] : sys.maxint}
        else:
            for i in range(0, len(self.filenames)):  #Wait for the find_cluster processes to end and get the results
                predMatrices[i] = self.finish_solving_word(i)
        return predMatrices

    def start_solving_word(self, filename):
        self.filenames.append(filename)

    def finish_solving_word(self, index):
        """Finishes solving a single word of a captcha using phash, by parsing the results of find_cluster"""
        answer = ""
        full_file = self.filenames[index].split("_")[0] + ".mp3"
        while answer == "" or answer == ";":
            if answer == "":
                self.play_file(self.filenames[index])
            else:
                self.play_file(full_file)
            answer = raw_input(self.filenames[index] + ") Enter the word you hear.  (Hit enter to replay it, ';' to listen to the whole thing)")
        return { answer : sys.maxint }    
        
    def play_file(self, filename, skip = ""):
        if self.mplayer != None:
            self.mplayer.kill()
        options = ["mplayer", filename]
        if skip != "":
            options.extend(["-ss", str(skip)])
        self.mplayer = subprocess.Popen(options, stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE)

if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="Solves a single word of a captcha using the human method.")
    parser.add_argument('-f', nargs='*', required=True, type=str, action='append', help='The wav/mp3 file(s) of the single word to solve')
    parser.add_argument('-v', required=False, action='store_true', help='Verbose mode.')
    parser.add_argument('-vv', required=False, action='store_true', help='Very Verbose mode.')
    args = parser.parse_args()    

    debug_level = logging.INFO
    if args.v or args.vv:
        debug_level = logging.DEBUG

    human_solver = HumanSolver(debug = debug_level)
    for filenames in args.f:
      for filename in filenames:
        startTime = time.time()

        human_solver.reset_solver()
        human_solver.start_solving_word(filename)
        matrix = human_solver.finish_solving_word(0)
        answer = max(matrix.iteritems(), key=operator.itemgetter(1))[0]

        if args.v or args.vv:
            print "Human determined",filename,"has an answer of",answer,"in",time.time()-startTime,"seconds."
        else:
            print filename + "," + answer

