#! /bin/octave -qf

arg_list = argv ();
if nargin < 8
    printf('Usage: octave trainNN3.m inputs hidden iterations save_every name_prefix inputx inputy output');
    exit();
end

%% Setup the parameters you will use for this exercise
input_layer_size  = str2num(arg_list{1});
hidden_layer_size = str2num(arg_list{2});
iterations = str2num(arg_list{3});
name_prefix = arg_list{4};
save_every = str2num(arg_list{5});    % never save in between ones
inputx = arg_list{6};
inputy = arg_list{7};
outputmat = arg_list{8};
num_labels = 10;          % 58 labels, from 1 to 58 (inclusive, 0 is unused)

printf("Loading training values\n");
% Load Training Data
% X, each row consists of pixel values for a single input
X = csvread(inputx);
% m, number of samples
m = size(X, 1);
% y, each row is the index of the word which corresponds to the example in X
printf("Loading answer values\n");
y = csvread(inputy);

% Initialize theta values
% Theta1, takes us from the input layer to the hidden layer
initial_Theta1 = randInitializeWeights(input_layer_size, hidden_layer_size);
% Theta2, takes us from the hidden layer to the output layer
initial_Theta2 = randInitializeWeights(hidden_layer_size, num_labels);
% Unroll parameters
initial_nn_params = [initial_Theta1(:) ; initial_Theta2(:)];

% Train algorith with the training set
options = optimset('MaxIter', iterations);
lambda = 0;
% Create "short hand" for the cost function to be minimized
costFunction = @(p) nnCostFunction(p, ...
                                   input_layer_size, ...
                                   hidden_layer_size, ...
                                   num_labels, X, y, lambda);
% Now, costFunction is a function that takes in only one argument (the
% neural network parameters)
printf("Minimizing cost\n");
[nn_params, cost] = fmincg3(costFunction, initial_nn_params, options, input_layer_size, hidden_layer_size, num_labels, name_prefix, save_every);
% Obtain Theta1 and Theta2 back from nn_params
Theta1 = reshape(nn_params(1:hidden_layer_size * (input_layer_size + 1)), ...
                 hidden_layer_size, (input_layer_size + 1));
Theta2 = reshape(nn_params((1 + (hidden_layer_size * (input_layer_size + 1))):end), ...
                 num_labels, (hidden_layer_size + 1));

save("-binary", outputmat, "Theta1", "Theta2");

