#!/usr/bin/python
#Dependencies: sox libsox-fmt-all octave curl
#Additionally, if you're on python2.6: python-argparse python-imaging

import sys, time, os, argparse, logging, shutil
import fuckit_splitter
import solver
from parse_solver_args import parse_solver_options,print_solver_help_and_exit

parser = argparse.ArgumentParser(description="Runs the complete captcha solving process against all captchas in a directory.")

#Solver options
parser.add_argument('-s', nargs='*', action='append', help='Adds a solver to the list of solvers to be used to solve the captcha. ' 
                    + 'This option can be specified multiple times to use additional solvers as backup.  Use -sh for information on specifying solvers')
parser.add_argument('-sh', required=False, action='store_true', help='Show the syntax to specify solvers')

#Misc args
parser.add_argument('dirname', help='The directory containing known good words in the form "captcha_##-123456789012.mp3" for a catcha with answer 123456789012')
parser.add_argument('-a', required=False, action='store_true', help='Save debug images and audio files to help with debugging the splitter')
parser.add_argument('-number_needed', type=int, default=11, help='The number needed to consider that we got a captcha correct')
parser.add_argument('-v', required=False, action='store_true', help='Verbose mode')
parser.add_argument('-vv', required=False, action='store_true', help='Very Verbose mode')

args = parser.parse_args()
if args.sh:
    print_solver_help_and_exit()

output_level = logging.CRITICAL
if args.vv:
    output_level = logging.DEBUG
elif args.v:
    output_level = logging.INFO

masher = None
splitter = fuckit_splitter.FuckItSplitter(debug = output_level)
solver_list, solver_thresholds = parse_solver_options(args.s, output_level)
captcha_solver = solver.CaptchaSolver(solver_list, splitter, [None], solver_thresholds, masher, output_level)

correct = 0
total = 0
correct_word_count = 0
total_word_count = 0
offline_files = os.listdir(args.dirname)
solved_files = []
for filename in offline_files:
    if filename.endswith(".mp3"):
        solved_files.append(filename)

for orig_filename in solved_files:
    total += 1
    filename = "current" + str(os.getpid())
    shutil.copy(args.dirname + "/" + orig_filename, captcha_solver.tempdir + "/" + filename + ".mp3")

    answer = captcha_solver.solve(filename)
    real_answer = orig_filename.split("-")[1][0:-4].replace("_", " ")
    print "Solved: ",answer
    print "Real:   ",real_answer
    if answer == "":
        print "No answer found for",filename
        success = False
    else:
        answer_parts = answer.split(" ")
        real_answer_parts = real_answer.replace(" ", "").replace("_","")
        correct_words = 0
        new_words = min(12, len(answer_parts), len(real_answer_parts))
        for i in range(0, new_words):
            total_word_count += 1
            if answer_parts[i] == real_answer_parts[i]:
                correct_words += 1
        correct_word_count += correct_words
        success = correct_words >= args.number_needed
        if success:
            correct += 1
            print "Correct",
        else:
            print "Incorrect",
        print "(" + str(correct_words) + "/" + str(new_words) + " right)"
        captcha_solver.delete_files(success, filename)

    print "Got ",correct,"/",total," = ",round(float(correct)/float(total) * 100, 4),"%  ",
    print correct_word_count,"/",total_word_count," words = ",round(float(correct_word_count)/float(total_word_count) * 100, 4), "%"

print "Final word accuracy:"
print float(correct_word_count)/float(total_word_count)
