
#include <stdio.h>
#include <dirent.h>
#include <errno.h>
#include <vector>
#include <map>
#include <string>
#include <algorithm>
#include <iostream>
#include <fstream>
#include <sys/stat.h>
#include <float.h>
#include "pHash.h"

using namespace std;

bool isPngFile(string filename)
{
	int length = filename.length();
	if(length < 4)
		return false;
	return filename.compare(length - 4, 4, ".png") == 0;
}

int loadHashesFromFile(string filename, map<string, vector<ulong64> *> & hashesPerDirectory, map<ulong64, string> & filePerHash)
{
	string line, cluster_dir, cluster_hash, cluster_file;
	int commaIndex, secondComma;
	char * dummy;
	ulong64 hash;

	ifstream hashFile(filename.c_str());
	if (hashFile.is_open()) {
		while(hashFile.good()) {
			getline (hashFile,line);
			commaIndex = line.find(",");
			if(commaIndex != string::npos) {
				secondComma = line.find(",", commaIndex + 1);
				cluster_dir = line.substr(0,commaIndex);

				if(secondComma == string::npos) {
					cluster_hash = line.substr(commaIndex + 1);
					cluster_file = "";
				}
				else {
					cluster_hash = line.substr(commaIndex + 1, secondComma - (commaIndex + 1));
					cluster_file = line.substr(secondComma + 1);
				}
				
				hash = strtoull(cluster_hash.c_str(), &dummy, 10);
				if(hashesPerDirectory.find(cluster_dir) == hashesPerDirectory.end())
					hashesPerDirectory[cluster_dir] = new vector<ulong64>();
				if(cluster_file != "")
					filePerHash[hash] = cluster_file;
				hashesPerDirectory[cluster_dir]->push_back(hash);
			}
		}
		hashFile.close();
	}
	else
		return -1;
	return 0;
}


int generateHashes(string directory_name, map<string, vector<ulong64> *> & hashesPerDirectory, map<ulong64, string> & filePerHash, int verbosity)
{
	ulong64 hash;
	DIR *dir, *cldir;
	struct dirent *ent, *clent;
	string clusterfile, clusterdir, fullclusterdir;
	ofstream hashFile("hashes"); 

  dir = opendir (directory_name.c_str());
	if (dir == NULL)
		return -1;

	while ((ent = readdir (dir)) != NULL) {
		if(strcmp(ent->d_name, ".") != 0 && strcmp(ent->d_name,"..") != 0) {
			clusterdir = ent->d_name;
			fullclusterdir = directory_name + clusterdir + "/";
			cldir = opendir(fullclusterdir.c_str());
			if(cldir == NULL)
				continue;
			
			hashesPerDirectory[clusterdir] = new vector<ulong64>();	
			while((clent = readdir(cldir)) != NULL) {
				if(strcmp(clent->d_name, ".") != 0 && strcmp(clent->d_name, "..") != 0) {
					clusterfile = directory_name + clusterdir + "/" + clent->d_name;
					if(isPngFile(clusterfile)){
						if (ph_dct_imagehash(clusterfile.c_str(), hash) >= 0){ //calculate the hash
							filePerHash[hash] = clusterfile;
							hashesPerDirectory[clusterdir]->push_back(hash);
							hashFile << clusterdir << "," << hash;
							if(verbosity == 0)
								hashFile << "," << clusterfile;
							hashFile << "\n";
						}
						else
							cout << "Could not calculate the hash for " << clusterfile << "\n";
					}
				}
			}
			closedir(cldir);
		}
	}
	closedir (dir);
	hashFile.close();
}

void cleanup(map<string, vector<ulong64> *> hashesPerDirectory)
{
	map<string, vector<ulong64> *>::iterator mit;
	for ( mit= hashesPerDirectory.begin(); mit != hashesPerDirectory.end(); mit++)
		free(mit->second);
}

string find_cluster_lowest(ulong64 hash, map<string, vector<ulong64> *>  & hashesPerDirectory, map<ulong64, string> filePerHash, int verbosity, int numberOfLowest)
{
	string solution = "nocluster";
	int sum = 0, best = INT_MAX;
	map<string, vector<ulong64> *>::iterator mit;
	map<string, vector<ulong64> *> distancesPerHash;
	vector<ulong64>::iterator vit;

	for (mit = hashesPerDirectory.begin(); mit != hashesPerDirectory.end(); mit++) 
	{
		distancesPerHash[mit->first] =  new vector<ulong64>(); 
		for (vit = mit->second->begin(); vit != mit->second->end(); vit++)
			distancesPerHash[mit->first]->push_back(ph_hamming_distance(hash, *vit));
	}

	for (mit = distancesPerHash.begin(); mit != distancesPerHash.end(); mit++) 
	{
		sum = 0;
		for(int i = 0; i < numberOfLowest; i++)
		{
			int lowest = 999999;
			int lowestIndex = -1;
			for (int j = 0; j < mit->second->size(); j++)
			{
				if((*mit->second)[j] < lowest)
				{
					lowest = (*mit->second)[j];
					lowestIndex = j;
				}
			}
			sum += lowest;
			if(lowestIndex != -1) //Could not find a file for this round
			{
				mit->second->erase(mit->second->begin() + lowestIndex);
				if(verbosity > 1)
					cout << lowest << " : " << mit->first << " : " << filePerHash[(*hashesPerDirectory[mit->first])[lowestIndex]] << "\n";
			}
			
		}
		if(sum < best) {
			best = sum; 
			solution = mit->first;	
		}
		if(verbosity > 0)
			cout << sum << ":" << mit->first << "\n";
	}

	cleanup(distancesPerHash);

	return solution;
}

int main(int argc, char **argv){
	map<string, vector<ulong64> *> hashesPerDirectory;
	map<ulong64, string> filePerHash;
	ulong64 hash;
	string solution;
	int verbosity = 0;
	string hash_file, cluster_directory;
	int numberOfLowest = 5;

	if (argc < 4) {
		cout << "Usage: find_cluster filename (-d cluster_directory| -f hashes_file) -v -m [-l number_of_lowest]  \n";
		return 1;
	}

	for(int i = 2; i < argc; i++)
	{
		if(strcmp(argv[i],"-v") == 0)
			verbosity++;
		else if(strcmp(argv[i],"-f") == 0 && i + 1 < argc)
			hash_file = argv[i + 1];
		else if(strcmp(argv[i],"-d") == 0 && i + 1 < argc)
			cluster_directory = argv[i + 1];
		else if(strcmp(argv[i],"-l") == 0 && i + 1 < argc)
			numberOfLowest = atoi(argv[i + 1]);
	}

	if(cluster_directory != "") {
		if(generateHashes(cluster_directory, hashesPerDirectory, filePerHash, verbosity) < 0) {
			cout << "Could not generate hashes for directory \"" << cluster_directory << "\"\n";
			return 1;
		}
	}
	else if(loadHashesFromFile(hash_file, hashesPerDirectory, filePerHash) < 0) {
		cout << "Unable to open hash file\n";
		return 1;
	}

	if (ph_dct_imagehash(argv[1], hash) < 0) {
		cout << "Could not calculate the hash for " << argv[1] << "\n";
		return 1;
	}
	
	solution = find_cluster_lowest(hash, hashesPerDirectory, filePerHash, verbosity, numberOfLowest);
	if(verbosity == 0)
		cout << solution << "\n";

	cleanup(hashesPerDirectory);

	return 0;
} 














